from __future__ import annotations

import json
from collections.abc import Mapping
from pathlib import Path
from typing import Any

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


from diffsync import Adapter, DiffSyncModel

from infrahub_sync import (
    DiffSyncMixin,
    DiffSyncModelMixin,
    SchemaMappingModel,
    SyncAdapter,
    SyncConfig,
)

"""Example custom adapter for demonstration purposes.

This adapter simulates connecting to a mock database containing Cars and People.
It demonstrates the key patterns for creating adapters in infrahub-sync.
"""


class MockDBClient:
    """Client for interacting with a mock database stored as a JSON file."""

    def __init__(self, filepath: str | None = None) -> None:
        """Initialize the mock database client.

        Args:
            filepath: Path to the mock database JSON file.
        """
        # Default to a sample database with a few entries
        self.data = {}

        # If a filepath is provided, try to load the data from it
        if filepath and Path(filepath).exists():
            try:
                with Path(filepath).open(encoding="utf-8") as f:
                    self.data = json.load(f)
                print(f"DEBUG: Loaded mock database from {filepath}")
            except (FileNotFoundError, json.JSONDecodeError) as e:
                print(f"DEBUG: Failed to load mock database from {filepath}: {e}")

    def get_all_nodes(self, model: str) -> list[dict[str, Any]]:
        """Get all nodes of a specific model from the mock database.

        Args:
            model: The name of the model to get nodes for.

        Returns:
            A list of dictionaries representing the nodes.
        """
        print(f"DEBUG: Getting all {model} nodes from mock database")
        nodes = self.data.get("nodes", {}).get(model, [])
        print(f"Loading {len(nodes)} {model} nodes")
        return nodes


class MockdbAdapter(DiffSyncMixin, Adapter):
    """A custom adapter that connects to a mock database."""

    type = "MockDB"

    def __init__(self, target: str, adapter: SyncAdapter, config: SyncConfig, *args, **kwargs) -> None:  # noqa: ANN002, ANN003
        """Initialize the custom adapter.

        Args:
            target: The target of the sync operation.
            adapter: The adapter configuration.
            config: The sync configuration.
        """
        super().__init__(*args, **kwargs)

        self.target = target
        self.config = config
        self.settings = adapter.settings or {}

        # Initialize the mock database client
        db_path = self.settings.get("db_path", "")
        self.client = MockDBClient(filepath=db_path or None)

        print(f"DEBUG: Initialized {self.__class__.__name__} adapter with target: {target}")
        print(f"DEBUG: Settings: {self.settings}")

    def model_loader(self, model_name: str, model: MockdbModel) -> None:
        """
        Load data for a specific model.

        This method is called by the DiffSync framework when loading
        data for a specific model.

        Args:
            model_name: The name of the model to load data for.
            model: The model class to instantiate.
        """
        print(f"DEBUG: Loading {model_name} data via model_loader...")

        # Find the corresponding schema mapping for this model
        for element in self.config.schema_mapping:
            if element.name == model_name:
                # Get the resource name from the mapping
                resource_name = element.mapping
                print(f"DEBUG: Found mapping for {model_name}: {resource_name}")

                # Get all nodes of this type from the mock database
                nodes = self.client.get_all_nodes(resource_name)
                total = len(nodes)

                # Apply filters if this is the source adapter
                if self.config.source.name.title() == self.type.title():
                    # Filter and transform the records
                    filtered_objs = model.filter_records(records=nodes, schema_mapping=element)
                    print(f"DEBUG: {self.type}: Loading {len(filtered_objs)}/{total} {model_name}")
                    transformed_objs = model.transform_records(records=filtered_objs, schema_mapping=element)
                else:
                    # No filtering needed
                    print(f"{self.type}: Loading all {total} {model_name}")
                    transformed_objs = nodes

                # Add each object to the adapter
                for obj in transformed_objs:
                    data = self.obj_to_diffsync(obj=obj, mapping=element, model=model)
                    print(f"DEBUG: Adding {data.get('name', data.get('local_id'))} to adapter")
                    self.add(model(**data))

                # We found and processed the mapping, so we're done
                break

    def obj_to_diffsync(self, obj: dict[str, Any], mapping: SchemaMappingModel, model: MockdbModel) -> dict[str, Any]:
        """Transform an object from the mock database to a format compatible with DiffSync.

        Args:
            obj: The object from the mock database.
            mapping: The schema mapping to use.
            model: The model class.

        Returns:
            A dictionary containing the transformed data.
        """
        # Use name or id as the local ID
        obj_id = obj.get("name") or obj.get("id")
        data: dict[str, Any] = {"local_id": str(obj_id)}

        # Process each field in the mapping
        if mapping.fields:
            for field in mapping.fields:
                if field.static:
                    # Use the static value if provided
                    data[field.name] = field.static
                elif field.mapping and not field.reference:
                    # Map directly from the source object
                    value = obj.get(field.mapping)
                    if value is not None:
                        data[field.name] = value
                elif field.mapping and field.reference:
                    # Handle references to other objects
                    ref_model = field.reference
                    all_refs = self.store.get_all(model=ref_model)

                    if model.is_list(field.name):
                        # Handle list references
                        peers = []
                        for ref_id in obj.get(field.mapping, []):
                            matched = [ref for ref in all_refs if ref.local_id == str(ref_id)]
                            if matched:
                                peers.append(matched[0].get_unique_id())
                        data[field.name] = peers
                    else:
                        # Handle single references
                        ref_id = obj.get(field.mapping)
                        if ref_id is not None:
                            matched = [ref for ref in all_refs if ref.local_id == str(ref_id)]
                            if matched:
                                data[field.name] = matched[0].get_unique_id()

        return data


class MockdbModel(DiffSyncModelMixin, DiffSyncModel):
    """A custom model for demonstration purposes."""

    @classmethod
    def create(
        cls,
        adapter: Adapter,
        ids: Mapping[Any, Any],
        attrs: Mapping[Any, Any],
    ) -> Self | None:
        # TODO: To implement
        return super().create(adapter=adapter, ids=ids, attrs=attrs)

    def update(self, attrs: dict) -> Self | None:
        # TODO: To implement
        return super().update(attrs=attrs)



from __future__ import annotations

from infrahub_sync.plugin_loader import PluginLoader

from .sync_models import (
    InfraAutonomousSystem,
    InfraBGPCommunity,
    InfraBGPPeerGroup,
    InfraBGPRoutingPolicy,
    InfraIXP,
    InfraIXPConnection,
    IpamIPAddress,
    OrganizationProvider,
)

# Load adapter class dynamically at runtime

_AdapterBaseClass = PluginLoader().resolve("peeringmanager")


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class PeeringmanagerSync(_AdapterBaseClass):
    IpamIPAddress = IpamIPAddress
    OrganizationProvider = OrganizationProvider
    InfraAutonomousSystem = InfraAutonomousSystem
    InfraBGPPeerGroup = InfraBGPPeerGroup
    InfraBGPCommunity = InfraBGPCommunity
    InfraBGPRoutingPolicy = InfraBGPRoutingPolicy
    InfraIXP = InfraIXP
    InfraIXPConnection = InfraIXPConnection



from __future__ import annotations

from typing import Any

from infrahub_sync.adapters.infrahub import InfrahubModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class VirtualizationVMDisk(InfrahubModel):
    _modelname = "VirtualizationVMDisk"
    _identifiers = ("virtual_machine", "device")
    _attributes = ("rotational",)
    device: str
    rotational: bool | None = None
    virtual_machine: str

    local_id: str | None = None
    local_data: Any | None = None


class VirtualizationVMFilesystem(InfrahubModel):
    _modelname = "VirtualizationVMFilesystem"
    _identifiers = ("virtual_machine", "mountpoint")
    _attributes = ("readonly", "fstype", "size_bytes", "device_error", "device")
    readonly: bool | None = None
    fstype: str | None = None
    size_bytes: int | None = None
    device_error: bool | None = None
    device: str | None = None
    mountpoint: str
    virtual_machine: str

    local_id: str | None = None
    local_data: Any | None = None


class VirtualizationVMNetworkInterface(InfrahubModel):
    _modelname = "VirtualizationVMNetworkInterface"
    _identifiers = ("virtual_machine", "name")
    _attributes = ("mtu", "mac_address", "carrier", "operational_status", "speed_bps", "duplex")
    mtu: int | None = None
    mac_address: str | None = None
    carrier: bool | None = None
    operational_status: str | None = None
    speed_bps: int | None = None
    duplex: str | None = None
    name: str
    virtual_machine: str

    local_id: str | None = None
    local_data: Any | None = None


class VirtualizationVirtualMachine(InfrahubModel):
    _modelname = "VirtualizationVirtualMachine"
    _identifiers = ("name",)
    _attributes = (
        "os_name",
        "ip_forwarding_enabled",
        "os_kernel",
        "status",
        "architecture",
        "conntrack_limit",
        "mem_total_bytes",
    )
    os_name: str | None = None
    ip_forwarding_enabled: bool | None = None
    os_kernel: str | None = None
    status: str | None = None
    architecture: str | None = None
    conntrack_limit: int | None = None
    name: str
    mem_total_bytes: int | None = None

    local_id: str | None = None
    local_data: Any | None = None



from __future__ import annotations

from typing import Any

from infrahub_sync.adapters.nautobot import NautobotModel

# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------


class CoreStandardGroup(NautobotModel):
    _modelname = "CoreStandardGroup"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class BuiltinTag(NautobotModel):
    _modelname = "BuiltinTag"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraAutonomousSystem(NautobotModel):
    _modelname = "InfraAutonomousSystem"
    _identifiers = ("name",)
    _attributes = ("organization", "description")
    name: str
    asn: int
    description: str | None = None
    organization: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraCircuit(NautobotModel):
    _modelname = "InfraCircuit"
    _identifiers = ("circuit_id",)
    _attributes = ("provider", "type", "tags", "description", "vendor_id")
    circuit_id: str
    description: str | None = None
    vendor_id: str | None = None
    provider: str
    type: str
    tags: list[str] | None

    local_id: str | None = None
    local_data: Any | None = None


class InfraDevice(NautobotModel):
    _modelname = "InfraDevice"
    _identifiers = ("location", "organization", "name")
    _attributes = (
        "model",
        "rack",
        "role",
        "tags",
        "platform",
        "serial_number",
        "asset_tag",
    )
    name: str | None = None
    serial_number: str | None = None
    asset_tag: str | None = None
    location: str
    model: str
    rack: str | None = None
    role: str | None = None
    tags: list[str] | None
    platform: str | None = None
    organization: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraFrontPort(NautobotModel):
    _modelname = "InfraFrontPort"
    _identifiers = ("name", "device")
    _attributes = ("rear_port", "description", "port_type")
    name: str
    description: str | None = None
    port_type: str | None = None
    rear_port: list[str] | None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraIPAddress(NautobotModel):
    _modelname = "InfraIPAddress"
    _identifiers = ("address", "vrf")
    _attributes = ("organization", "role", "description")
    address: str
    description: str | None = None
    organization: str | None = None
    vrf: str | None = None
    role: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraInterfaceL2L3(NautobotModel):
    _modelname = "InfraInterfaceL2L3"
    _identifiers = ("name", "device")
    _attributes = (
        "tagged_vlan",
        "tags",
        "l2_mode",
        "description",
        "mgmt_only",
        "mac_address",
        "interface_type",
    )
    l2_mode: str | None = None
    name: str
    description: str | None = None
    mgmt_only: bool | None = False
    mac_address: str | None = None
    interface_type: str | None = None
    untagged_vlan: str | None = None
    tagged_vlan: list[str] | None
    device: str
    tags: list[str] | None

    local_id: str | None = None
    local_data: Any | None = None


class InfraPlatform(NautobotModel):
    _modelname = "InfraPlatform"
    _identifiers = ("name", "manufacturer")
    _attributes = ("description", "napalm_driver")
    name: str
    description: str | None = None
    napalm_driver: str | None = None
    manufacturer: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraPrefix(NautobotModel):
    _modelname = "InfraPrefix"
    _identifiers = ("prefix", "vrf", "organization")
    _attributes = ("role", "vlan", "description")
    prefix: str
    description: str | None = None
    organization: str | None = None
    role: str | None = None
    vrf: str | None = None
    vlan: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraProviderNetwork(NautobotModel):
    _modelname = "InfraProviderNetwork"
    _identifiers = ("name",)
    _attributes = ("provider", "tags", "description", "vendor_id")
    name: str
    description: str | None = None
    vendor_id: str | None = None
    provider: str
    tags: list[str] | None

    local_id: str | None = None
    local_data: Any | None = None


class InfraRack(NautobotModel):
    _modelname = "InfraRack"
    _identifiers = ("name",)
    _attributes = (
        "location",
        "role",
        "tags",
        "height",
        "facility_id",
        "serial_number",
        "asset_tag",
    )
    name: str
    height: int | None = None
    facility_id: str | None = None
    serial_number: str | None = None
    asset_tag: str | None = None
    location: str
    role: str | None = None
    tags: list[str] | None

    local_id: str | None = None
    local_data: Any | None = None


class InfraRearPort(NautobotModel):
    _modelname = "InfraRearPort"
    _identifiers = ("name", "device")
    _attributes = ("description", "port_type")
    name: str
    description: str | None = None
    port_type: str | None = None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraRouteTarget(NautobotModel):
    _modelname = "InfraRouteTarget"
    _identifiers = ("name", "organization")
    _attributes = ("description",)
    name: str
    description: str | None = None
    organization: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVLAN(NautobotModel):
    _modelname = "InfraVLAN"
    _identifiers = ("name", "vlan_id", "location", "organization")
    _attributes = ("role", "vlan_group", "description")
    name: str
    description: str | None = None
    vlan_id: int
    organization: str | None = None
    role: str | None = None
    vlan_group: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVRF(NautobotModel):
    _modelname = "InfraVRF"
    _identifiers = ("name",)
    _attributes = ("organization", "import_rt", "export_rt", "description", "vrf_rd")
    name: str
    description: str | None = None
    vrf_rd: str | None = None
    organization: str | None = None
    import_rt: list[str] | None
    export_rt: list[str] | None

    local_id: str | None = None
    local_data: Any | None = None


class LocationGeneric(NautobotModel):
    _modelname = "LocationGeneric"
    _identifiers = ("name",)
    _attributes = ("organization", "location_type", "description", "type")
    name: str
    description: str | None = None
    type: str
    organization: str | None = None
    location_type: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class RoleGeneric(NautobotModel):
    _modelname = "RoleGeneric"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class StatusGeneric(NautobotModel):
    _modelname = "StatusGeneric"
    _identifiers = ("name",)
    _attributes = ("label", "description")
    name: str
    label: str | None = None
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceCircuitType(NautobotModel):
    _modelname = "ChoiceCircuitType"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceDeviceType(NautobotModel):
    _modelname = "ChoiceDeviceType"
    _identifiers = ("name", "manufacturer")
    _attributes = ("tags", "part_number", "height", "full_depth")
    part_number: str | None = None
    height: int | None = None
    full_depth: bool | None = None
    name: str
    manufacturer: str
    tags: list[str] | None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceLocationType(NautobotModel):
    _modelname = "ChoiceLocationType"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None



from __future__ import annotations

from infrahub_sync.plugin_loader import PluginLoader

from .sync_models import (
    DcimPhysicalDevice,
    DcimPhysicalInterface,
    LocationBuilding,
    LocationMetro,
    OrganizationCustomer,
)

# Load adapter class dynamically at runtime

_AdapterBaseClass = PluginLoader().resolve("aci")


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class AciSync(_AdapterBaseClass):
    DcimPhysicalDevice = DcimPhysicalDevice
    DcimPhysicalInterface = DcimPhysicalInterface
    LocationBuilding = LocationBuilding
    LocationMetro = LocationMetro
    OrganizationCustomer = OrganizationCustomer



"""Replacement for Makefile."""

from invoke import Collection, Context, task

from . import docs, linter, tests

ns = Collection("infrahub_sync")
ns.configure(
    {
        "infrahub_sync": {
            "project_name": "infrahub_sync",
            "python_ver": "3.10",
            "local": False,
        }
    }
)
ns.add_collection(linter)
ns.add_collection(docs)
ns.add_collection(tests)


@task(name="lint")
def lint_all(context: Context) -> None:
    docs.lint(context)
    linter.lint_all(context)


@task(name="format")
def format_all(context: Context) -> None:
    docs.format(context)
    linter.format_all(context)


@task(name="tests-all")
def test_all(context: Context) -> None:
    tests.tests_unit(context)
    tests.tests_integration(context)


@task(name="tests-unit")
def tests_unit(context: Context) -> None:
    tests.tests_unit(context)


@task(name="tests-integration")
def tests_integration(context: Context) -> None:
    tests.tests_integration(context)


@task(name="generate-doc")
def generate_doc(context: Context) -> None:
    docs.generate_doc(context)


@task(name="docusaurus")
def docusaurus(context: Context) -> None:
    docs.docusaurus(context)


ns.add_task(lint_all)
ns.add_task(format_all)
ns.add_task(test_all)
ns.add_task(tests_unit)
ns.add_task(tests_integration)
ns.add_task(generate_doc)
ns.add_task(docusaurus)



import sys
from pathlib import Path

from invoke import Context, task

from .utils import ESCAPED_REPO_PATH, check_if_command_available

NAMESPACE = "INFRAHUB-SYNC-DOCS"
CURRENT_DIRECTORY = Path(__file__).parent.resolve()
DOCUMENTATION_DIRECTORY = CURRENT_DIRECTORY.parent / "docs"


@task
def generate(context: Context) -> None:
    """Generate documentation for the infrahub-sync cli."""
    _generate_infrahubsync_documentation(context=context)


def _generate_infrahubsync_documentation(context: Context) -> None:
    """Generate the documentation for infrahub-sync using typer-cli."""

    print(" - Generate infrahub-sync CLI documentation")
    exec_cmd = 'poetry run typer infrahub_sync.cli utils docs --name "infrahub-sync"'
    exec_cmd += " --output docs/docs/reference/cli.mdx"
    with context.cd(ESCAPED_REPO_PATH):
        context.run(exec_cmd)


@task
def markdownlint(context: Context) -> None:
    has_markdownlint = check_if_command_available(context=context, command_name="markdownlint-cli2")

    if not has_markdownlint:
        print("Warning, markdownlint-cli2 is not installed")
        return
    exec_cmd = "markdownlint-cli2 **/*.{md,mdx} '#**/node_modules/**'"
    print(" - [docs] Lint docs with markdownlint-cli2")
    with context.cd(ESCAPED_REPO_PATH):
        context.run(exec_cmd)


@task
def format_markdownlint(context: Context) -> None:
    """Run markdownlint-cli2 to format all .md/mdx files."""

    print(" - [docs] Format code with markdownlint-cli2")
    exec_cmd = "markdownlint-cli2 **/*.{md,mdx} --fix"
    with context.cd(ESCAPED_REPO_PATH):
        context.run(exec_cmd)


@task
def format(context: Context) -> None:  # noqa: A001
    """This will run all formatters."""
    format_markdownlint(context)


@task
def lint(context: Context) -> None:
    """This will run all linters."""
    markdownlint(context)


@task
def docusaurus(context: Context) -> None:
    """Build documentation website."""
    exec_cmd = "npm run build"

    with context.cd(DOCUMENTATION_DIRECTORY):
        output = context.run(exec_cmd)

    if output.exited != 0:
        sys.exit(-1)



from __future__ import annotations

from typing import Any

from infrahub_sync.plugin_loader import PluginLoader

# Load model class dynamically at runtime (honor adapters_path, safe fallback)
try:
    _loader = PluginLoader.from_env_and_args(adapter_paths=[])

    _spec = "infrahub"

    _ModelBaseClass = _loader.resolve(_spec, default_class_candidates=("Model",))
except Exception:
    # Fallback: use DiffSyncModel to avoid import-time failure
    from diffsync import DiffSyncModel as _FallbackModel

    _ModelBaseClass = _FallbackModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class CoreStandardGroup(_ModelBaseClass):
    _modelname = "CoreStandardGroup"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class BuiltinTag(_ModelBaseClass):
    _modelname = "BuiltinTag"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceCircuitType(_ModelBaseClass):
    _modelname = "ChoiceCircuitType"
    _identifiers = ("name",)
    _attributes = ("tags", "description")
    description: str | None = None
    name: str
    tags: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceDeviceType(_ModelBaseClass):
    _modelname = "ChoiceDeviceType"
    _identifiers = ("name", "manufacturer")
    _attributes = ("tags", "height", "full_depth", "part_number")
    height: int | None = None
    full_depth: bool | None = None
    part_number: str | None = None
    name: str
    manufacturer: str
    tags: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class InfraCircuit(_ModelBaseClass):
    _modelname = "InfraCircuit"
    _identifiers = ("circuit_id",)
    _attributes = ("tags", "type", "provider", "vendor_id", "description")
    circuit_id: str
    vendor_id: str | None = None
    description: str | None = None
    tags: list[str] | None = []
    type: str
    provider: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraDevice(_ModelBaseClass):
    _modelname = "InfraDevice"
    _identifiers = ("location", "rack", "organization", "name")
    _attributes = ("model", "role", "tags", "serial_number", "description", "asset_tag")
    name: str | None = None
    serial_number: str | None = None
    description: str | None = None
    asset_tag: str | None = None
    model: str
    rack: str | None = None
    role: str | None = None
    tags: list[str] | None = []
    location: str
    organization: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraIPAddress(_ModelBaseClass):
    _modelname = "InfraIPAddress"
    _identifiers = ("address", "vrf")
    _attributes = ("organization", "description")
    address: str
    description: str | None = None
    organization: str | None = None
    vrf: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraInterfaceL2L3(_ModelBaseClass):
    _modelname = "InfraInterfaceL2L3"
    _identifiers = ("device", "name")
    _attributes = ("tagged_vlan", "tags", "l2_mode", "mac_address", "description", "interface_type", "mgmt_only")
    l2_mode: str | None = None
    mac_address: str | None = None
    description: str | None = None
    interface_type: str | None = None
    mgmt_only: bool | None = False
    name: str
    tagged_vlan: list[str] | None = []
    untagged_vlan: str | None = None
    tags: list[str] | None = []
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraPrefix(_ModelBaseClass):
    _modelname = "InfraPrefix"
    _identifiers = ("prefix", "vrf")
    _attributes = ("role", "organization", "location", "description")
    prefix: str
    description: str | None = None
    role: str | None = None
    organization: str | None = None
    location: str | None = None
    vrf: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraProviderNetwork(_ModelBaseClass):
    _modelname = "InfraProviderNetwork"
    _identifiers = ("name",)
    _attributes = ("provider", "tags", "vendor_id", "description")
    vendor_id: str | None = None
    name: str
    description: str | None = None
    provider: str
    tags: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class InfraRack(_ModelBaseClass):
    _modelname = "InfraRack"
    _identifiers = ("name", "location")
    _attributes = ("tags", "role", "asset_tag", "facility_id", "height", "serial_number")
    asset_tag: str | None = None
    facility_id: str | None = None
    height: int | None = None
    name: str
    serial_number: str | None = None
    tags: list[str] | None = []
    role: str | None = None
    location: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraRouteTarget(_ModelBaseClass):
    _modelname = "InfraRouteTarget"
    _identifiers = ("name", "organization")
    _attributes = ("description",)
    name: str
    description: str | None = None
    organization: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVLAN(_ModelBaseClass):
    _modelname = "InfraVLAN"
    _identifiers = ("name", "vlan_id", "location", "vlan_group")
    _attributes = ("organization", "description")
    description: str | None = None
    name: str
    vlan_id: int
    vlan_group: str | None = None
    organization: str | None = None
    location: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVRF(_ModelBaseClass):
    _modelname = "InfraVRF"
    _identifiers = ("name",)
    _attributes = ("export_rt", "organization", "import_rt", "description", "vrf_rd")
    description: str | None = None
    vrf_rd: str | None = None
    name: str
    export_rt: list[str] | None = []
    organization: str | None = None
    import_rt: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class LocationGeneric(_ModelBaseClass):
    _modelname = "LocationGeneric"
    _identifiers = ("name",)
    _attributes = ("organization", "tags", "group", "description", "type")
    description: str | None = None
    type: str
    name: str
    organization: str | None = None
    tags: list[str] | None = []
    group: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class OrganizationGeneric(_ModelBaseClass):
    _modelname = "OrganizationGeneric"
    _identifiers = ("name",)
    _attributes = ("group", "description")
    description: str | None = None
    name: str
    group: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class RoleGeneric(_ModelBaseClass):
    _modelname = "RoleGeneric"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None



from pathlib import Path

from invoke import Context, UnexpectedExit

path = Path(__file__)
TASKS_DIR = path.parent
REPO_BASE = TASKS_DIR.parent


def check_if_command_available(context: Context, command_name: str) -> bool:
    try:
        context.run(f"command -v {command_name}", hide=True)
    except UnexpectedExit:
        return False
    return True


def escape_path(path: Path) -> str:
    """Escape special characters in the provided path string to make it shell-safe."""
    return str(path).translate(
        str.maketrans(
            {
                "-": r"\-",
                "]": r"\]",
                "\\": r"\\",
                "^": r"\^",
                "$": r"\$",
                "*": r"\*",
                "(": r"\(",
                ")": r"\)",
                ".": r"\.",
            }
        )
    )


ESCAPED_REPO_PATH = escape_path(REPO_BASE)



from __future__ import annotations

from typing import Any

from infrahub_sync.plugin_loader import PluginLoader

# Load model class dynamically at runtime (honor adapters_path, safe fallback)
try:
    _loader = PluginLoader.from_env_and_args(adapter_paths=[])

    _spec = "infrahub"

    _ModelBaseClass = _loader.resolve(_spec, default_class_candidates=("Model",))
except Exception:
    # Fallback: use DiffSyncModel to avoid import-time failure
    from diffsync import DiffSyncModel as _FallbackModel

    _ModelBaseClass = _FallbackModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class CoreStandardGroup(_ModelBaseClass):
    _modelname = "CoreStandardGroup"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraDevice(_ModelBaseClass):
    _modelname = "InfraDevice"
    _identifiers = ("name",)
    _attributes = ("platform", "primary_address", "description", "type")
    description: str | None = None
    type: str
    name: str
    platform: str | None = None
    primary_address: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class IpamIPAddress(_ModelBaseClass):
    _modelname = "IpamIPAddress"
    _identifiers = ("address",)
    _attributes = ("description",)
    address: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None



from infrahub_sync.adapters.slurpitsync import SlurpitsyncAdapter

from .sync_models import (
    ChoiceDeviceType,
    InfraDevice,
    InfraHardwareInfo,
    InfraInterface,
    InfraIPAddress,
    InfraPlatform,
    InfraPrefix,
    InfraVersion,
    InfraVLAN,
    InfraVRF,
    LocationGeneric,
    OrganizationGeneric,
)


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class SlurpitsyncSync(SlurpitsyncAdapter):
    InfraDevice = InfraDevice
    InfraHardwareInfo = InfraHardwareInfo
    InfraIPAddress = InfraIPAddress
    InfraInterface = InfraInterface
    InfraPlatform = InfraPlatform
    InfraPrefix = InfraPrefix
    InfraVLAN = InfraVLAN
    InfraVRF = InfraVRF
    InfraVersion = InfraVersion
    LocationGeneric = LocationGeneric
    OrganizationGeneric = OrganizationGeneric
    ChoiceDeviceType = ChoiceDeviceType



from __future__ import annotations

from typing import Any

from infrahub_sync.plugin_loader import PluginLoader

# Load model class dynamically at runtime (honor adapters_path, safe fallback)
try:
    _loader = PluginLoader.from_env_and_args(adapter_paths=[])

    _spec = "genericrestapi"

    _ModelBaseClass = _loader.resolve(_spec, default_class_candidates=("Model",))
except Exception:
    # Fallback: use DiffSyncModel to avoid import-time failure
    from diffsync import DiffSyncModel as _FallbackModel

    _ModelBaseClass = _FallbackModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class BuiltinTag(_ModelBaseClass):
    _modelname = "BuiltinTag"
    _identifiers = ("name",)
    _attributes = ()
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class LocationSite(_ModelBaseClass):
    _modelname = "LocationSite"
    _identifiers = ("name",)
    _attributes = ("tags",)
    name: str
    tags: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class OrganizationTenant(_ModelBaseClass):
    _modelname = "OrganizationTenant"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None



"""Cisco ACI API."""

from __future__ import annotations

import json
import logging
import os
from datetime import datetime, timedelta, timezone
from typing import TYPE_CHECKING, Any, ClassVar

import requests  # type: ignore[import]
import urllib3
from diffsync import Adapter, DiffSyncModel
from requests import Response  # type: ignore[import]
from requests.adapters import HTTPAdapter  # type: ignore[import]
from typing_extensions import Self
from urllib3.util.retry import Retry  # type: ignore[import]

from infrahub_sync import (
    DiffSyncMixin,
    DiffSyncModelMixin,
    SchemaMappingModel,
    SyncAdapter,
    SyncConfig,
)

from .utils import get_value

if TYPE_CHECKING:
    import builtins

logger = logging.getLogger(__name__)


class InvalidAciCredentialError(ValueError):
    """Exception raised for invalid ACI credentials."""

    def __init__(self, field: str) -> None:
        # Accept the field name (recommended) and produce a consistent message
        msg = f"{field} must be a non-empty string"
        super().__init__(msg)


class AciApiClient:
    """Representation and methods for interacting with aci."""

    def __init__(
        self,
        username: str,
        password: str,
        base_url: str,
        verify: bool | str | None = True,
    ) -> None:
        """Initialization of AciApiClient.

        Args:
          username: APIC username
          password: APIC password
          base_url: Base URL of the APIC API (must include trailing slash handling)
          verify: SSL verification flag. Accepts bool or string values (for env parsing).
        """
        # Basic input validation to fail fast and provide clear errors
        if not isinstance(username, str) or not username:
            field_name = "username"
            raise InvalidAciCredentialError(field_name)
        if not isinstance(password, str) or not password:
            field_name = "password"
            raise InvalidAciCredentialError(field_name)
        if not isinstance(base_url, str) or not base_url:
            field_name = "base_url"
            raise InvalidAciCredentialError(field_name)

        # Normalize verify to a bool (support string env values like "false", "0")
        # Normalize verify to a boolean. Accepts strings like "false", "0", "no".
        self.verify = verify.lower() not in ("0", "false", "no") if isinstance(verify, str) else bool(verify)

        self.username = username
        self.password = password
        self.base_url = base_url
        self.cookies = ""
        self.last_login: datetime | None = None
        self.refresh_timeout: int | None = None
        # Suppress TLS warnings only when verification is disabled
        if not self.verify:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        # Use a requests Session for connection pooling and to preserve cookies
        self.session: requests.Session = requests.Session()
        # Configure retries with backoff for idempotent operations and common transient errors
        retry_strategy = Retry(
            total=3,
            backoff_factor=0.3,
            status_forcelist=[429, 500, 502, 503, 504],
            allowed_methods=["HEAD", "GET", "OPTIONS"],
        )
        adapter = HTTPAdapter(max_retries=retry_strategy)
        self.session.mount("https://", adapter)
        self.session.mount("http://", adapter)

    def _login(self) -> Response:
        """Method to log into the ACI fabric and retrieve the token."""
        payload = {"aaaUser": {"attributes": {"name": self.username, "pwd": self.password}}}
        url = self.base_url + "aaaLogin.json"
        resp = self._handle_request(url, request_type="post", data=payload)
        if resp.ok:
            self.cookies = resp.cookies
            # Store last login time as UTC to avoid timezone related attribute errors
            self.last_login = datetime.now(tz=timezone.utc)
            self.refresh_timeout = int(resp.json()["imdata"][0]["aaaLogin"]["attributes"]["refreshTimeoutSeconds"])
        return resp

    def _handle_request(
        self,
        url: str,
        params: dict | None = None,
        request_type: str = "get",
        data: dict | None = None,
    ) -> Response:
        """Send a REST API call to the APIC."""
        try:
            resp = self.session.request(
                method=request_type.upper(),
                url=url,
                cookies=self.cookies,
                params=params,
                verify=self.verify,
                json=data,
                timeout=30,
            )
            resp.raise_for_status()
        except requests.exceptions.HTTPError as http_err:
            # Log and propagate HTTP errors with context (include traceback)
            msg = f"HTTP error when calling {url}: {http_err}"
            logger.exception(msg)
            raise
        except requests.exceptions.ConnectionError as conn_err:
            msg = f"Connection error when calling {url}: {conn_err}"
            logger.exception(msg)
            raise ConnectionError(msg) from conn_err
        except requests.exceptions.RequestException as req_err:
            # Catch-all for other requests exceptions (timeout, too many redirects, etc.)
            msg = f"Request failed when calling {url}: {req_err}"
            logger.exception(msg)
            raise ConnectionError(msg) from req_err
        return resp

    def _request(self, method: str, uri: str, params: dict | None = None, data: dict | None = None) -> Response:
        """Helper to perform authenticated requests. Ensures login/refresh before sending."""
        url = self.base_url + uri
        # Ensure we are logged in; if login fails we'll raise in _handle_error
        if self._refresh_token():
            login_resp = self._login()
            if not login_resp.ok:
                self._handle_error(login_resp)

        resp = self._handle_request(url, params=params, request_type=method, data=data)
        if resp.ok:
            return resp
        self._handle_error(resp)
        # This line should never be reached since _handle_error raises
        msg = "Unhandled error in AciApiClient._request"
        raise requests.exceptions.HTTPError(msg)

    def _refresh_token(self) -> bool:
        """Private method to check if the login token needs refreshed.
        Returns: True if login needs refresh."""
        if not self.last_login or not self.refresh_timeout:
            # If we've never logged in or refresh timeout is unavailable, force login
            return True
        # Compare using UTC-aware datetimes
        return datetime.now(tz=timezone.utc) - self.last_login > timedelta(seconds=self.refresh_timeout)

    def _handle_error(self, response: Response) -> None:
        """Private method to handle HTTP errors by raising an HTTPError.

        Args:
          response: requests.Response instance triggering the error
        """
        msg = (
            f"There was an HTTP error while performing operation on {self.base_url}: "
            f"Error: {response.status_code}, Reason: {response.reason}"
        )
        logger.error(msg)
        raise requests.exceptions.HTTPError(msg)

    def get(self, uri: str, params: dict | None = None) -> Response:
        """Retrieve data from the ACI fabric and return the raw Response."""
        return self._request("get", uri, params=params)

    def post(self, uri: str, params: dict | None = None, data: dict | None = None) -> Response:
        """Post data to the ACI fabric and return the raw Response."""
        return self._request("post", uri, params=params, data=data)


class AciAdapter(DiffSyncMixin, Adapter):
    """Adapter for Cisco ACI API"""

    type = "CiscoAci"

    def __init__(self, target: str, adapter: SyncAdapter, config: SyncConfig, *args: Any, **kwargs: Any) -> None:
        """Initialize aci adapter"""
        super().__init__(*args, **kwargs)
        self.target = target
        self.client = self._create_aci_client(adapter=adapter)
        self.config = config
        # Build device mapping once during initialization
        self.device_mapping = self._build_device_mapping()
        AciModel.set_device_mapping(self.device_mapping)

    def _create_aci_client(self, adapter: SyncAdapter) -> AciApiClient:
        settings = adapter.settings or {}
        url = os.environ.get("CISCO_APIC_URL") or settings.get("url")
        username = os.environ.get("CISCO_APIC_USERNAME") or settings.get("username")
        password = os.environ.get("CISCO_APIC_PASSWORD") or settings.get("password")
        # Prefer explicit env var for verify; allow boolean or string values
        verify_raw = os.environ.get("CISCO_APIC_VERIFY")
        if verify_raw is None:
            verify_raw = settings.get("verify", True)
        verify = verify_raw.lower() not in ("0", "false", "no") if isinstance(verify_raw, str) else bool(verify_raw)
        api_endpoint = settings.get("api_endpoint", "api")  # Default endpoint, change if necessary

        if not url:
            msg = "url must be specified!"
            raise ValueError(msg)
        if not username or not password:
            msg = "username and password must be specified!"
            raise ValueError(msg)

        full_base_url = f"{url.rstrip('/')}/{api_endpoint.rstrip('/')}/"
        return AciApiClient(
            base_url=full_base_url,
            username=str(username),
            password=str(password),
            verify=verify,
        )

    def model_loader(self, model_name: str, model: builtins.type[AciModel]) -> None:  # type: ignore[valid-type]
        """
        Load and process models using schema mapping filters and transformations.

        This method retrieves data from ACI, applies filters and transformations
        as specified in the schema mapping, and loads the processed data into the adapter.
        """
        # Retrieve schema mapping for this model
        for element in self.config.schema_mapping:
            if element.name != model_name:
                continue
            if not element.fields:
                logger.debug("No fields defined for schema mapping %s, skipping", element.name)
                continue

            if not element.mapping:
                print(f"No mapping defined for '{element.name}', skipping...")
                continue

            # Use the resource endpoint from the schema mapping
            resource_name = element.mapping

            try:
                # Retrieve all objects
                response_data = self.client.get(resource_name)
                objs = response_data.json().get("imdata", [])
            except Exception as exc:
                msg = f"Error fetching data from REST API: {exc!s}"
                raise ValueError(msg) from exc

            logger.debug(json.dumps(objs, indent=2))
            total = len(objs)
            # Always apply filters and transforms for the source adapter
            # Check if this is the source adapter by comparing source name with adapter type
            is_source_adapter = self.config.source.name.lower() == "aci"
            if is_source_adapter:
                # Filter records
                filtered_objs = model.filter_records(records=objs, schema_mapping=element)  # type: ignore[attr-defined]
                logger.info("%s: Loading %d/%d %s", self.type, len(filtered_objs), total, resource_name)
                # Transform records
                transformed_objs = model.transform_records(records=filtered_objs, schema_mapping=element)  # type: ignore[attr-defined]
            else:
                logger.info("%s: Loading all %d %s", self.type, total, resource_name)
                transformed_objs = objs

            # Create model instances after filtering and transforming
            for obj in transformed_objs:
                data = self.obj_to_diffsync(obj=obj, mapping=element, model=model)
                item = model(**data)  # type: ignore[misc]
                self.add(item)

    def obj_to_diffsync(
        self,
        obj: dict[str, Any],
        mapping: SchemaMappingModel,
        model: builtins.type[AciModel],  # type: ignore[valid-type]
    ) -> dict[str, Any]:
        """Convert an object to DiffSync format based on the provided mapping schema."""
        obj_id = self._extract_aci_id(obj)

        data: dict[str, Any] = {"local_id": str(obj_id) if obj_id else ""}

        if not mapping.fields:
            return data
        for field in mapping.fields:
            field_is_list = model.is_list(name=field.name)  # type: ignore[attr-defined]

            if field.static:
                data[field.name] = field.static
            elif field.mapping and not field.reference:
                value = get_value(obj, field.mapping)
                if value is not None:
                    data[field.name] = value
            elif field.mapping and field.reference:
                all_nodes_for_reference = self.store.get_all(model=field.reference)
                nodes = list(all_nodes_for_reference)

                if not nodes and all_nodes_for_reference:
                    msg = (
                        f"Unable to get '{field.mapping}' with '{field.reference}' reference from store. "
                        f"The available models are {self.store.get_all_model_names()}"
                    )
                    raise ValueError(msg)

                if field_is_list:
                    data[field.name] = self._process_list_field(obj, field, nodes)
                else:
                    data[field.name] = self._process_single_field(obj, field, nodes)
        logger.debug("Converted object to diffsync data: %s", data)
        return data

    def _process_list_field(self, obj: dict[str, Any], field: Any, nodes: list[Any]) -> list[str]:
        """Process a list field and return a list of unique IDs."""
        unique_ids = []
        nodes_in_obj = get_value(obj, field.mapping) or []
        for node in nodes_in_obj:
            if not node:
                continue
            node_id = str(node) if node else ""
            matching_nodes = [item for item in nodes if item.local_id == node_id]
            if not matching_nodes:
                msg = f"Unable to locate the node {field.reference} {node_id}"
                raise ValueError(msg)
            unique_ids.append(matching_nodes[0].get_unique_id())
        return sorted(unique_ids)

    def _process_single_field(self, obj: dict[str, Any], field: Any, nodes: list[Any]) -> str | None:
        """Process a single field and return its unique ID."""
        # Check if this field has already been transformed (e.g., by Jinja filters)
        if field.name in obj:
            # Use the already transformed value from the object
            node_value = str(obj[field.name])
        else:
            # Fall back to extracting from the original mapping
            node = get_value(obj, field.mapping)
            if not node:
                return None
            node_value = str(node)

        # For device references, try matching against both local_id and name
        # since devices might be referenced by name (due to identifier configuration)
        matching_nodes = [item for item in nodes if item.local_id == node_value]

        if not matching_nodes:
            # Try matching against the name field for device references
            matching_nodes = [item for item in nodes if getattr(item, "name", None) == node_value]

        if not matching_nodes:
            available_ids = [getattr(item, "local_id", "N/A") for item in nodes[:5]]
            available_names = [getattr(item, "name", "N/A") for item in nodes[:5]]
            msg = (
                f"Unable to locate the node {field.reference} {node_value} for field '{field.name}'. "
                f"Available node IDs (first 5): {available_ids}, "
                f"Available names (first 5): {available_names}"
            )
            logger.error(msg)
            raise ValueError(msg)

        return matching_nodes[0].get_unique_id()

    def _build_device_mapping(self) -> dict[str, str]:
        """Build device mapping from ACI fabricNode data during initialization."""
        device_mapping = {}

        try:
            # Query ACI for fabricNode data once
            response = self.client.get("class/fabricNode.json")
            fabric_nodes = response.json().get("imdata", [])

            # Build mapping from fabricNode data
            for obj in fabric_nodes:
                if "fabricNode" in obj:
                    fabric_node = obj["fabricNode"]
                    if isinstance(fabric_node, dict) and "attributes" in fabric_node:
                        attrs = fabric_node["attributes"]
                        node_id = attrs.get("id")
                        node_name = attrs.get("name")

                        if node_id and node_name:
                            device_mapping[str(node_id)] = node_name

            logger.info("Built ACI device mapping with %d entries", len(device_mapping))
            logger.debug("Device mapping: %s", device_mapping)

        except (requests.exceptions.RequestException, ValueError, KeyError) as e:
            logger.warning("Failed to build device mapping from fabricNode data: %s", e)

        return device_mapping

    def _extract_aci_id(self, obj: dict[str, Any]) -> str | None:
        """Extract ID from ACI objects, handling the nested structure."""
        if not obj:
            return None

        # Check for fabricNode structure first
        if "fabricNode" in obj:
            fabric_node = obj["fabricNode"]
            if isinstance(fabric_node, dict) and "attributes" in fabric_node:
                attrs = fabric_node["attributes"]
                node_id = attrs.get("id")
                if node_id:
                    return str(node_id)

        # Fallback to generic id field
        obj_id = obj.get("id")
        if obj_id:
            return str(obj_id)

        return None


class AciModel(DiffSyncModelMixin, DiffSyncModel):
    """ACI Model with ACI-specific Jinja filters."""

    # Class-level storage for device mapping (set by AciAdapter)
    _device_mapping: ClassVar[dict[str, str]] = {}

    @classmethod
    def set_device_mapping(cls, device_mapping: dict[str, str]) -> None:
        """Set the device mapping for ACI models."""
        cls._device_mapping = device_mapping

    @classmethod
    def _add_custom_filters(cls, native_env: Any, item: dict[str, Any]) -> None:  # noqa: ARG003
        """Add ACI-specific filters to the Jinja environment."""

        def aci_device_name(node_id: str) -> str:
            """
            Custom Jinja filter to resolve ACI node IDs to device names using pre-built mapping.

            Args:
                node_id: The ACI node ID (e.g., "101", "102")

            Returns:
                Device name if found, otherwise returns the original node_id
            """
            return cls._device_mapping.get(str(node_id), node_id)

        native_env.filters["aci_device_name"] = aci_device_name

    @classmethod
    def create(
        cls,
        adapter: Adapter,
        ids: dict[Any, Any],
        attrs: dict[Any, Any],
    ) -> Self | None:
        # TODO: To implement
        return super().create(adapter=adapter, ids=ids, attrs=attrs)

    def update(self, attrs: dict) -> Self | None:
        # TODO: To implement
        return super().update(attrs)



from infrahub_sync.adapters.infrahub import InfrahubAdapter

from .sync_models import (
    ChoiceDeviceType,
    InfraDevice,
    InfraInterfaceL3,
    InfraIPAddress,
    InfraNOSVersion,
    InfraPartNumber,
    InfraPlatform,
    InfraPrefix,
    InfraVLAN,
    InfraVRF,
    LocationGeneric,
    OrganizationGeneric,
)


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class InfrahubSync(InfrahubAdapter):
    InfraDevice = InfraDevice
    InfraIPAddress = InfraIPAddress
    InfraInterfaceL3 = InfraInterfaceL3
    InfraNOSVersion = InfraNOSVersion
    InfraPartNumber = InfraPartNumber
    InfraPlatform = InfraPlatform
    InfraPrefix = InfraPrefix
    InfraVLAN = InfraVLAN
    InfraVRF = InfraVRF
    LocationGeneric = LocationGeneric
    OrganizationGeneric = OrganizationGeneric
    ChoiceDeviceType = ChoiceDeviceType



from pathlib import Path

from invoke import Context, task

NAMESPACE = "INFRAHUB-SYNC-TEST"
CURRENT_DIRECTORY = Path(__file__).parent.resolve()
MAIN_DIRECTORY = CURRENT_DIRECTORY.parent

# ----------------------------------------------------------------------------
# Tests tasks
# ----------------------------------------------------------------------------


@task
def tests_unit(context: Context) -> None:
    pass


@task
def tests_integration(context: Context) -> None:
    pass



from __future__ import annotations

from typing import Any

from infrahub_sync.adapters.observium import ObserviumModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class CoreStandardGroup(ObserviumModel):
    _modelname = "CoreStandardGroup"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraDevice(ObserviumModel):
    _modelname = "InfraDevice"
    _identifiers = ("name",)
    _attributes = ("platform", "primary_address", "type", "description")
    type: str
    name: str
    description: str | None = None
    platform: str | None = None
    primary_address: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class IpamIPAddress(ObserviumModel):
    _modelname = "IpamIPAddress"
    _identifiers = ("address",)
    _attributes = ("description",)
    description: str | None = None
    address: str

    local_id: str | None = None
    local_data: Any | None = None



from __future__ import annotations

from typing import Any

from infrahub_sync.adapters.infrahub import InfrahubModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class InfraDevice(InfrahubModel):
    _modelname = "InfraDevice"
    _identifiers = ("hostname",)
    _attributes = (
        "model",
        "location",
        "platform",
        "version",
        "fqdn",
        "serial_number",
        "hardware_serial_number",
    )
    fqdn: str | None = None
    hostname: str
    serial_number: str
    hardware_serial_number: str
    model: str | None = None
    location: str
    platform: str | None = None
    version: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraIPAddress(InfrahubModel):
    _modelname = "InfraIPAddress"
    _identifiers = ("interface", "address")
    _attributes = ("prefix", "description")
    address: str
    description: str | None = None
    interface: str | None = None
    prefix: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraInterfaceL3(InfrahubModel):
    _modelname = "InfraInterfaceL3"
    _identifiers = ("device", "name")
    _attributes = ("description", "speed", "mtu", "mac_address")
    name: str
    description: str | None = None
    speed: int | None = None
    mtu: int | None = 1500
    mac_address: str | None = None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraNOSVersion(InfrahubModel):
    _modelname = "InfraNOSVersion"
    _identifiers = ("manufacturer", "model", "version")
    _attributes = ("platform",)
    version: str
    manufacturer: str
    platform: str | None = None
    model: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraPartNumber(InfrahubModel):
    _modelname = "InfraPartNumber"
    _identifiers = ("device", "name")
    _attributes = (
        "model",
        "manufacturer",
        "part_vid",
        "part_id",
        "description",
        "part_sn",
    )
    name: str
    part_vid: str | None = None
    part_id: str | None = None
    description: str | None = None
    part_sn: str | None = None
    device: str
    model: str | None = None
    manufacturer: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraPlatform(InfrahubModel):
    _modelname = "InfraPlatform"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraPrefix(InfrahubModel):
    _modelname = "InfraPrefix"
    _identifiers = ("vrf", "prefix")
    _attributes = ("vlan", "location")
    prefix: str
    vlan: str | None = None
    vrf: str | None = None
    location: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVLAN(InfrahubModel):
    _modelname = "InfraVLAN"
    _identifiers = ("location", "vlan_id")
    _attributes = ("description", "name")
    vlan_id: int
    description: str | None = None
    name: str | None = None
    location: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraVRF(InfrahubModel):
    _modelname = "InfraVRF"
    _identifiers = ("name",)
    _attributes = ("vrf_rd",)
    name: str
    vrf_rd: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class LocationGeneric(InfrahubModel):
    _modelname = "LocationGeneric"
    _identifiers = ("name",)
    _attributes = ("type", "description")
    type: str
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class OrganizationGeneric(InfrahubModel):
    _modelname = "OrganizationGeneric"
    _identifiers = ("name",)
    _attributes = ("type",)
    name: str
    type: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceDeviceType(InfrahubModel):
    _modelname = "ChoiceDeviceType"
    _identifiers = ("name", "manufacturer")
    _attributes = ("description",)
    description: str | None = None
    name: str
    manufacturer: str | None = None

    local_id: str | None = None
    local_data: Any | None = None



from infrahub_sync.adapters.nautobot import NautobotAdapter

from .sync_models import (
    BuiltinTag,
    ChoiceCircuitType,
    ChoiceDeviceType,
    ChoiceLocationType,
    CoreStandardGroup,
    InfraAutonomousSystem,
    InfraCircuit,
    InfraDevice,
    InfraFrontPort,
    InfraInterfaceL2L3,
    InfraIPAddress,
    InfraPlatform,
    InfraPrefix,
    InfraProviderNetwork,
    InfraRack,
    InfraRearPort,
    InfraRouteTarget,
    InfraVLAN,
    InfraVRF,
    LocationGeneric,
    RoleGeneric,
    StatusGeneric,
)


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class NautobotSync(NautobotAdapter):
    CoreStandardGroup = CoreStandardGroup
    BuiltinTag = BuiltinTag
    InfraAutonomousSystem = InfraAutonomousSystem
    InfraCircuit = InfraCircuit
    InfraDevice = InfraDevice
    InfraFrontPort = InfraFrontPort
    InfraIPAddress = InfraIPAddress
    InfraInterfaceL2L3 = InfraInterfaceL2L3
    InfraPlatform = InfraPlatform
    InfraPrefix = InfraPrefix
    InfraProviderNetwork = InfraProviderNetwork
    InfraRack = InfraRack
    InfraRearPort = InfraRearPort
    InfraRouteTarget = InfraRouteTarget
    InfraVLAN = InfraVLAN
    InfraVRF = InfraVRF
    LocationGeneric = LocationGeneric
    RoleGeneric = RoleGeneric
    StatusGeneric = StatusGeneric
    ChoiceCircuitType = ChoiceCircuitType
    ChoiceDeviceType = ChoiceDeviceType
    ChoiceLocationType = ChoiceLocationType



from __future__ import annotations

from typing import Any

from infrahub_sync.plugin_loader import PluginLoader

# Load model class dynamically at runtime (honor adapters_path, safe fallback)
try:
    _loader = PluginLoader.from_env_and_args(adapter_paths=[])

    _spec = "infrahub"

    _ModelBaseClass = _loader.resolve(_spec, default_class_candidates=("Model",))
except Exception:
    # Fallback: use DiffSyncModel to avoid import-time failure
    from diffsync import DiffSyncModel as _FallbackModel

    _ModelBaseClass = _FallbackModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class CoreStandardGroup(_ModelBaseClass):
    _modelname = "CoreStandardGroup"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class BuiltinTag(_ModelBaseClass):
    _modelname = "BuiltinTag"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraAutonomousSystem(_ModelBaseClass):
    _modelname = "InfraAutonomousSystem"
    _identifiers = ("name",)
    _attributes = ("organization", "description", "asn")
    name: str
    description: str | None = None
    asn: int
    organization: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraCircuit(_ModelBaseClass):
    _modelname = "InfraCircuit"
    _identifiers = ("circuit_id",)
    _attributes = ("status", "type", "tags", "provider", "description", "vendor_id")
    circuit_id: str
    description: str | None = None
    vendor_id: str | None = None
    status: str | None = None
    type: str
    tags: list[str] | None = []
    provider: str

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceCircuitType(_ModelBaseClass):
    _modelname = "ChoiceCircuitType"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraDevice(_ModelBaseClass):
    _modelname = "InfraDevice"
    _identifiers = ("location", "organization", "name")
    _attributes = ("role", "model", "status", "platform", "rack", "tags", "asset_tag", "serial_number")
    name: str | None = None
    asset_tag: str | None = None
    serial_number: str | None = None
    role: str | None = None
    model: str
    status: str | None = None
    location: str
    platform: str | None = None
    organization: str | None = None
    rack: str | None = None
    tags: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceDeviceType(_ModelBaseClass):
    _modelname = "ChoiceDeviceType"
    _identifiers = ("name", "manufacturer")
    _attributes = ("tags", "part_number", "full_depth", "height")
    part_number: str | None = None
    full_depth: bool | None = None
    height: int | None = None
    name: str
    manufacturer: str
    tags: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class InfraFrontPort(_ModelBaseClass):
    _modelname = "InfraFrontPort"
    _identifiers = ("name", "device")
    _attributes = ("rear_port", "description", "port_type")
    name: str
    description: str | None = None
    port_type: str | None = None
    rear_port: list[str] | None = []
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraInterfaceL2L3(_ModelBaseClass):
    _modelname = "InfraInterfaceL2L3"
    _identifiers = ("name", "device")
    _attributes = (
        "tagged_vlan",
        "tags",
        "status",
        "l2_mode",
        "description",
        "mac_address",
        "mgmt_only",
        "interface_type",
    )
    l2_mode: str | None = None
    description: str | None = None
    name: str
    mac_address: str | None = None
    mgmt_only: bool | None = False
    interface_type: str | None = None
    tagged_vlan: list[str] | None = []
    untagged_vlan: str | None = None
    device: str
    tags: list[str] | None = []
    status: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraIPAddress(_ModelBaseClass):
    _modelname = "InfraIPAddress"
    _identifiers = ("address", "ip_prefix")
    _attributes = ("organization", "interfaces", "role", "status", "description")
    description: str | None = None
    address: str
    organization: str | None = None
    interfaces: list[str] | None = []
    role: str | None = None
    status: str | None = None
    ip_prefix: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceLocationType(_ModelBaseClass):
    _modelname = "ChoiceLocationType"
    _identifiers = ("name",)
    _attributes = ("description",)
    name: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class NautobotNamespace(_ModelBaseClass):
    _modelname = "NautobotNamespace"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraPlatform(_ModelBaseClass):
    _modelname = "InfraPlatform"
    _identifiers = ("name", "manufacturer")
    _attributes = ("napalm_driver", "description")
    napalm_driver: str | None = None
    name: str
    description: str | None = None
    manufacturer: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraProviderNetwork(_ModelBaseClass):
    _modelname = "InfraProviderNetwork"
    _identifiers = ("name",)
    _attributes = ("provider", "tags", "status", "description", "vendor_id")
    description: str | None = None
    name: str
    vendor_id: str | None = None
    provider: str
    tags: list[str] | None = []
    status: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraPrefix(_ModelBaseClass):
    _modelname = "InfraPrefix"
    _identifiers = ("prefix", "ip_namespace")
    _attributes = ("organization", "locations", "status", "role", "vlan", "description")
    description: str | None = None
    prefix: str
    organization: str | None = None
    locations: list[str] | None = []
    status: str | None = None
    role: str | None = None
    vlan: str | None = None
    ip_namespace: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraRack(_ModelBaseClass):
    _modelname = "InfraRack"
    _identifiers = ("name",)
    _attributes = ("role", "tags", "location", "serial_number", "asset_tag", "height", "facility_id")
    name: str
    serial_number: str | None = None
    asset_tag: str | None = None
    height: int | None = None
    facility_id: str | None = None
    role: str | None = None
    tags: list[str] | None = []
    location: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraRearPort(_ModelBaseClass):
    _modelname = "InfraRearPort"
    _identifiers = ("name", "device")
    _attributes = ("description", "port_type")
    name: str
    description: str | None = None
    port_type: str | None = None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraRouteTarget(_ModelBaseClass):
    _modelname = "InfraRouteTarget"
    _identifiers = ("name", "organization")
    _attributes = ("description",)
    name: str
    description: str | None = None
    organization: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVLAN(_ModelBaseClass):
    _modelname = "InfraVLAN"
    _identifiers = ("name",)
    _attributes = ("role", "vlan_group", "locations", "organization", "status", "description", "vlan_id")
    name: str
    description: str | None = None
    vlan_id: int
    role: str | None = None
    vlan_group: str | None = None
    locations: list[str] | None = []
    organization: str | None = None
    status: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVRF(_ModelBaseClass):
    _modelname = "InfraVRF"
    _identifiers = ("name", "ip_namespace")
    _attributes = ("import_rt", "export_rt", "organization", "vrf_rd", "description")
    name: str
    vrf_rd: str | None = None
    description: str | None = None
    ip_namespace: str
    import_rt: list[str] | None = []
    export_rt: list[str] | None = []
    organization: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class OrganizationGeneric(_ModelBaseClass):
    _modelname = "OrganizationGeneric"
    _identifiers = ("name",)
    _attributes = ("group", "description", "type")
    name: str
    description: str | None = None
    type: str | None = None
    group: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class StatusGeneric(_ModelBaseClass):
    _modelname = "StatusGeneric"
    _identifiers = ("name",)
    _attributes = ("label", "description")
    name: str
    label: str | None = None
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class RoleGeneric(_ModelBaseClass):
    _modelname = "RoleGeneric"
    _identifiers = ("name",)
    _attributes = ("label", "description")
    name: str
    label: str | None = None
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class LocationGeneric(_ModelBaseClass):
    _modelname = "LocationGeneric"
    _identifiers = ("name",)
    _attributes = ("tags", "location_type", "status", "description")
    name: str
    description: str | None = None
    tags: list[str] | None = []
    location_type: str | None = None
    status: str | None = None

    local_id: str | None = None
    local_data: Any | None = None



from infrahub_sync.adapters.prometheus import PrometheusAdapter

from .sync_models import (
    VirtualizationVirtualMachine,
    VirtualizationVMDisk,
    VirtualizationVMFilesystem,
    VirtualizationVMNetworkInterface,
)


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class PrometheusSync(PrometheusAdapter):
    VirtualizationVMDisk = VirtualizationVMDisk
    VirtualizationVMFilesystem = VirtualizationVMFilesystem
    VirtualizationVMNetworkInterface = VirtualizationVMNetworkInterface
    VirtualizationVirtualMachine = VirtualizationVirtualMachine



from pathlib import Path

from invoke import Context, task

from .utils import ESCAPED_REPO_PATH

NAMESPACE = "INFRAHUB-SYNC"
CURRENT_DIRECTORY = Path(__file__).parent.resolve()
MAIN_DIRECTORY = CURRENT_DIRECTORY.parent


@task(name="format")
def lint_all(context: Context) -> None:
    """This will run all linter."""

    lint_ruff(context)
    lint_pylint(context)
    lint_yaml(context)

    print(f" - [{NAMESPACE}] All linter have been executed!")


@task(name="format")
def format_all(context: Context) -> None:
    """This will run all formatter."""

    format_ruff(context)

    print(f" - [{NAMESPACE}] All formatters have been executed!")


# ----------------------------------------------------------------------------
# Linter tasks - Python
# ----------------------------------------------------------------------------
@task
def lint_pylint(context: Context) -> None:
    """This will run pylint for the specified name and Python version."""

    print(f" - [{NAMESPACE}] Check code with pylint")
    exec_cmd = f"pylint {MAIN_DIRECTORY}"
    with context.cd(ESCAPED_REPO_PATH):
        context.run(exec_cmd)


@task
def lint_ruff(context: Context) -> None:
    """This will run ruff."""

    print(f" - [{NAMESPACE}] Check code with ruff")
    exec_cmd = f"ruff format --check --diff {MAIN_DIRECTORY} &&"
    exec_cmd += f"ruff check --diff {MAIN_DIRECTORY}"
    with context.cd(ESCAPED_REPO_PATH):
        context.run(exec_cmd)


# ----------------------------------------------------------------------------
# Linter tasks - Yaml
# ----------------------------------------------------------------------------


@task
def lint_yaml(context: Context) -> None:
    """This will run yamllint to validate formatting of all yaml files."""

    print(f" - [{NAMESPACE}] Format yaml with yamllint")
    exec_cmd = "yamllint ."
    context.run(exec_cmd, pty=True)


# ----------------------------------------------------------------------------
# Formatting tasks - Python
# ----------------------------------------------------------------------------
@task
def format_ruff(context: Context) -> None:
    """This will run ruff."""

    print(f" - [{NAMESPACE}] Check code with ruff")
    exec_cmd = f"ruff format {MAIN_DIRECTORY} && "
    exec_cmd += f"ruff check --fix {MAIN_DIRECTORY}"
    with context.cd(ESCAPED_REPO_PATH):
        context.run(exec_cmd)



from __future__ import annotations

from typing import Any

from infrahub_sync.adapters.librenms import LibrenmsModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class CoreStandardGroup(LibrenmsModel):
    _modelname = "CoreStandardGroup"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraDevice(LibrenmsModel):
    _modelname = "InfraDevice"
    _identifiers = ("name",)
    _attributes = ("site", "type")
    type: str
    name: str
    site: str

    local_id: str | None = None
    local_data: Any | None = None


class IpamIPAddress(LibrenmsModel):
    _modelname = "IpamIPAddress"
    _identifiers = ("address",)
    _attributes = ("description",)
    address: str
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class LocationSite(LibrenmsModel):
    _modelname = "LocationSite"
    _identifiers = ("name",)
    _attributes = ("description",)
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None



from infrahub_sync.adapters.librenms import LibrenmsAdapter

from .sync_models import (
    CoreStandardGroup,
    InfraDevice,
    IpamIPAddress,
    LocationSite,
)


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class LibrenmsSync(LibrenmsAdapter):
    CoreStandardGroup = CoreStandardGroup
    InfraDevice = InfraDevice
    IpamIPAddress = IpamIPAddress
    LocationSite = LocationSite



from __future__ import annotations

from infrahub_sync.plugin_loader import PluginLoader

from .sync_models import (
    BuiltinTag,
    ChoiceCircuitType,
    ChoiceDeviceType,
    ChoiceLocationType,
    CoreStandardGroup,
    InfraAutonomousSystem,
    InfraCircuit,
    InfraDevice,
    InfraFrontPort,
    InfraInterfaceL2L3,
    InfraIPAddress,
    InfraPlatform,
    InfraPrefix,
    InfraProviderNetwork,
    InfraRack,
    InfraRearPort,
    InfraRouteTarget,
    InfraVLAN,
    InfraVRF,
    LocationGeneric,
    NautobotNamespace,
    OrganizationGeneric,
    RoleGeneric,
    StatusGeneric,
)

# Load adapter class dynamically at runtime

_AdapterBaseClass = PluginLoader().resolve("infrahub")


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class InfrahubSync(_AdapterBaseClass):
    CoreStandardGroup = CoreStandardGroup
    BuiltinTag = BuiltinTag
    InfraAutonomousSystem = InfraAutonomousSystem
    InfraCircuit = InfraCircuit
    ChoiceCircuitType = ChoiceCircuitType
    InfraDevice = InfraDevice
    ChoiceDeviceType = ChoiceDeviceType
    InfraFrontPort = InfraFrontPort
    InfraInterfaceL2L3 = InfraInterfaceL2L3
    InfraIPAddress = InfraIPAddress
    ChoiceLocationType = ChoiceLocationType
    NautobotNamespace = NautobotNamespace
    InfraPlatform = InfraPlatform
    InfraProviderNetwork = InfraProviderNetwork
    InfraPrefix = InfraPrefix
    InfraRack = InfraRack
    InfraRearPort = InfraRearPort
    InfraRouteTarget = InfraRouteTarget
    InfraVLAN = InfraVLAN
    InfraVRF = InfraVRF
    OrganizationGeneric = OrganizationGeneric
    StatusGeneric = StatusGeneric
    RoleGeneric = RoleGeneric
    LocationGeneric = LocationGeneric



from __future__ import annotations

from typing import Any

from infrahub_sync.plugin_loader import PluginLoader

# Load model class dynamically at runtime (honor adapters_path, safe fallback)
try:
    _loader = PluginLoader.from_env_and_args(adapter_paths=[])

    _spec = "peeringmanager"

    _ModelBaseClass = _loader.resolve(_spec, default_class_candidates=("Model",))
except Exception:
    # Fallback: use DiffSyncModel to avoid import-time failure
    from diffsync import DiffSyncModel as _FallbackModel

    _ModelBaseClass = _FallbackModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class InfraAutonomousSystem(_ModelBaseClass):
    _modelname = "InfraAutonomousSystem"
    _identifiers = ("asn",)
    _attributes = ("ipv4_max_prefixes", "ipv6_max_prefixes", "affiliated", "name", "irr_as_set", "description")
    ipv4_max_prefixes: int | None = None
    ipv6_max_prefixes: int | None = None
    affiliated: bool | None = None
    name: str
    asn: int
    irr_as_set: str | None = None
    description: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraBGPPeerGroup(_ModelBaseClass):
    _modelname = "InfraBGPPeerGroup"
    _identifiers = ("name",)
    _attributes = ("export_policies", "import_policies", "bgp_communities", "description", "status")
    description: str | None = None
    status: str | None = None
    name: str
    export_policies: list[str] | None = []
    import_policies: list[str] | None = []
    bgp_communities: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class InfraBGPCommunity(_ModelBaseClass):
    _modelname = "InfraBGPCommunity"
    _identifiers = ("name",)
    _attributes = ("value", "label", "community_type", "description")
    value: str
    label: str | None = None
    community_type: str | None = None
    description: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraBGPRoutingPolicy(_ModelBaseClass):
    _modelname = "InfraBGPRoutingPolicy"
    _identifiers = ("name",)
    _attributes = ("bgp_communities", "address_family", "description", "label", "policy_type", "weight")
    name: str
    address_family: int
    description: str | None = None
    label: str | None = None
    policy_type: str
    weight: int | None = 1000
    bgp_communities: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class InfraIXP(_ModelBaseClass):
    _modelname = "InfraIXP"
    _identifiers = ("name",)
    _attributes = ("import_policies", "export_policies", "bgp_communities", "status", "description")
    name: str
    status: str | None = "enabled"
    description: str | None = None
    import_policies: list[str] | None = []
    export_policies: list[str] | None = []
    bgp_communities: list[str] | None = []

    local_id: str | None = None
    local_data: Any | None = None


class InfraIXPConnection(_ModelBaseClass):
    _modelname = "InfraIXPConnection"
    _identifiers = ("name",)
    _attributes = ("internet_exchange_point", "peeringdb_netixlan", "vlan", "description", "status")
    name: str
    peeringdb_netixlan: int | None = None
    vlan: int | None = None
    description: str | None = None
    status: str | None = "enabled"
    internet_exchange_point: str

    local_id: str | None = None
    local_data: Any | None = None



from __future__ import annotations

from infrahub_sync.plugin_loader import PluginLoader

from .sync_models import (
    InfraAutonomousSystem,
)

# Load adapter class dynamically at runtime

_AdapterBaseClass = PluginLoader().resolve("infrahub")


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class InfrahubSync(_AdapterBaseClass):
    InfraAutonomousSystem = InfraAutonomousSystem



from __future__ import annotations

from typing import Any

from infrahub_sync.adapters.infrahub import InfrahubModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class InfraDevice(InfrahubModel):
    _modelname = "InfraDevice"
    _identifiers = ("hostname",)
    _attributes = ("manufacturer", "location", "device_type", "platform", "fqdn")
    hostname: str
    fqdn: str | None = None
    manufacturer: str | None = None
    location: str | None = None
    device_type: str | None = None
    platform: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraHardwareInfo(InfrahubModel):
    _modelname = "InfraHardwareInfo"
    _identifiers = ("device", "serial")
    _attributes = ("name", "description", "product", "version")
    name: str
    description: str | None = None
    product: str | None = None
    serial: str | None = None
    version: str | None = None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraIPAddress(InfrahubModel):
    _modelname = "InfraIPAddress"
    _identifiers = ("address", "ip_prefix")
    _attributes = ("interface",)
    address: str
    interface: str | None = None
    prefix: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraInterface(InfrahubModel):
    _modelname = "InfraInterface"
    _identifiers = ("device", "name")
    _attributes = ("description", "mac_address")
    name: str
    description: str | None = None
    mac_address: str | None = None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraPlatform(InfrahubModel):
    _modelname = "InfraPlatform"
    _identifiers = ("name",)
    _attributes = ()
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraPrefix(InfrahubModel):
    _modelname = "InfraPrefix"
    _identifiers = ("vrf", "prefix")
    _attributes = ()
    prefix: str
    vrf: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class InfraVLAN(InfrahubModel):
    _modelname = "InfraVLAN"
    _identifiers = ("vlan_id", "name")
    _attributes = ()
    name: str | None = None
    vlan_id: int

    local_id: str | None = None
    local_data: Any | None = None


class InfraVRF(InfrahubModel):
    _modelname = "InfraVRF"
    _identifiers = ("name",)
    _attributes = ()
    name: str

    local_id: str | None = None
    local_data: Any | None = None


class InfraVersion(InfrahubModel):
    _modelname = "InfraVersion"
    _identifiers = ("version",)
    _attributes = ("devices", "file")
    version: str
    devices: list[str] | None = []
    file: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class LocationGeneric(InfrahubModel):
    _modelname = "LocationGeneric"
    _identifiers = ("name",)
    _attributes = (
        "description",
        "number",
        "street",
        "city",
        "county",
        "state",
        "zipcode",
        "country",
        "phonenumber",
    )
    name: str
    description: str | None = None
    number: str | None = None
    street: str | None = None
    zipcode: str | None = None
    country: str | None = None
    phonenumber: str | None = None
    city: str | None = None
    county: str | None = None
    state: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class OrganizationGeneric(InfrahubModel):
    _modelname = "OrganizationGeneric"
    _identifiers = ("name",)
    _attributes = ("type",)
    name: str
    type: str | None = None

    local_id: str | None = None
    local_data: Any | None = None


class ChoiceDeviceType(InfrahubModel):
    _modelname = "ChoiceDeviceType"
    _identifiers = ("name",)
    _attributes = ("manufacturer",)
    name: str
    manufacturer: str | None = None

    local_id: str | None = None
    local_data: Any | None = None



from __future__ import annotations

from typing import Any

from infrahub_sync.plugin_loader import PluginLoader

# Load model class dynamically at runtime (honor adapters_path, safe fallback)
try:
    _loader = PluginLoader.from_env_and_args()

    _spec = "aci"

    _ModelBaseClass = _loader.resolve(_spec, default_class_candidates=("Model",))
except Exception:
    # Fallback: use DiffSyncModel to avoid import-time failure
    from diffsync import DiffSyncModel as _FallbackModel

    _ModelBaseClass = _FallbackModel


# -------------------------------------------------------
# AUTO-GENERATED FILE, DO NOT MODIFY
#  This file has been generated with the command `infrahub-sync generate`
#  All modifications will be lost the next time you reexecute this command
# -------------------------------------------------------
class DcimPhysicalDevice(_ModelBaseClass):
    _modelname = "DcimPhysicalDevice"
    _identifiers = ("name",)
    _attributes = ("location", "serial", "role", "description", "status")
    serial: str | None = None
    name: str
    role: str | None = None
    description: str | None = None
    status: str
    location: str

    local_id: str | None = None
    local_data: Any | None = None


class DcimPhysicalInterface(_ModelBaseClass):
    _modelname = "DcimPhysicalInterface"
    _identifiers = ("device", "name")
    _attributes = ("description",)
    name: str
    description: str | None = None
    device: str

    local_id: str | None = None
    local_data: Any | None = None


class LocationBuilding(_ModelBaseClass):
    _modelname = "LocationBuilding"
    _identifiers = ("name",)
    _attributes = ("parent", "shortname")
    name: str
    shortname: str
    parent: str

    local_id: str | None = None
    local_data: Any | None = None


class LocationMetro(_ModelBaseClass):
    _modelname = "LocationMetro"
    _identifiers = ("name",)
    _attributes = ("shortname",)
    name: str
    shortname: str

    local_id: str | None = None
    local_data: Any | None = None


class OrganizationCustomer(_ModelBaseClass):
    _modelname = "OrganizationCustomer"
    _identifiers = ("name", "customer_id")
    _attributes = ()
    customer_id: str | None = None
    name: str

    local_id: str | None = None
    local_data: Any | None = None
